#
#    Project : "Station de qualité de l'air Lora"
#
#    Copyright (C) 2019  Yohann LE GALL & Sergio QUINTERO
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.


import time
import utime
from machine import I2C
import adafruit_am2320
from machine import UART
import sds011

_FIELD_TEMPERATURE = 'temperature'
_FIELD_HUMIDITY = 'humidity'
_FIELD_PM25 = 'pm2.5'
_FIELD_PM10 = 'pm10'

# temp_measure_interval = 89000
hm_temp_measure_interval = 40000
hm_temp_last_measure = 0

dust_measure_interval = 35000
dust_last_measure = 0

def isTimeToTakeMeasure(sensorLastMeasure, sensorMeasureInterval):
    """
    Verifies if the measure interval of a given sensor is completed and returns True or False
    Arguments:\n
    :sensorLastMeasure: Time of the last measure taken in miliseconds\n
    :sensorMeasureInterval: Interval of time predefined between each measure in miliseconds
    """
    timeis= time.ticks_ms()
    diff = timeis - sensorLastMeasure

    # if ((sensorLastMeasure == 0) or (time.ticks_diff(sensorLastMeasure,time.ticks_ms()) >= sensorMeasureInterval)):
    if ((sensorLastMeasure == 0) or (diff >= sensorMeasureInterval)):
        return True
    return False

#message to be send at the end of the last send time
global_message = {}
#current measures taken before the last send interval is achieved
current_message = {}
message_info = 0

#SDS011 variables init
fan_status = False
fan_time_running = 0
meas_dur = 30000
last_meas_time = 0

# Initialize the I2C bus
i2c = I2C(0, I2C.MASTER)
i2c.init(I2C.MASTER, baudrate=100000)

#Initialize the AM2320 sensor
am = adafruit_am2320.AM2320(i2c)

# Initialize the UART
# uart_dust = UART(1, baudrate=9600, pins=('P22','P21'))
uart_dust = UART(1, baudrate=9600)

# Initialize the SDS011 sensor
sds011 = sds011.SDS011(uart_dust)
sds011.sleep()

#Number of reading tentatives, as this sensor does not always respond...
n_try_max = 10

while True:
    success = False
    n_try = 0

    pycom.rgbled(0xFF0000)  # Red
    time.sleep(1)
    pycom.rgbled(0x000000)

    current_message = {}

    # Measure of Temperature & Humidity
    try:
        if isTimeToTakeMeasure(hm_temp_last_measure, hm_temp_measure_interval):
            hm_temp_last_measure = time.ticks_ms()
            while(not success and n_try < n_try_max):
                try:
                    # print("Temperature: ", am.temperature)
                    current_message[_FIELD_TEMPERATURE] = am.temperature
                    # print("Humidity: ", am.relative_humidity)
                    current_message[_FIELD_HUMIDITY] = am.relative_humidity
                    message_info += 2

                    print("Current : " + str(current_message))

                    success = True
                except:
                    # These sensors are a bit flakey, its ok if the readings fail
                    n_try += 1

    except Exception as e:
        print("Temperature & Humidity sensor exception : " + str(e))

    # Measure dust PM
    try:
        if isTimeToTakeMeasure(dust_last_measure, dust_measure_interval):
            print("Time has passed")
            dust_last_measure = time.ticks_ms()
            if(fan_status is False):
                sds011.wake()
                fan_time_running = time.ticks_ms()
                fan_status = True

            timeis = time.ticks_ms()
            if (timeis - fan_time_running >= meas_dur):
                print("Fan dust ok")
                last_meas_time = time.ticks_ms()
                sds011.read()
                current_message[_FIELD_PM25] = sds011.pm25
                current_message[_FIELD_PM10] = sds011.pm10
                message_info += 2

                #Stop FAN
                sds011.sleep()
                fan_status = False

                print("Current : " + str(current_message))
                
    except Exception as e:
        print("DUST Sensor exception : " + str(e))

    #Probing this sensor too often makes it crash...
    #Let us do that once per second

   


    try:
        if(message_info > 0):
            # CONNEXION LORA ICI
            # message_info = 0
            # Date = str(utime.mktime(utime.localtime()))
            # global_message[Date] = current_message
            # print("Current : " + str(global_message))
    except Exception as e:
        print("Update current message exception : " + str(e))

    time.sleep(1)
